<?php
define('BACKUP_SUCCESS','13'); 
define('BACKUPSETFORDISPLAY','BackupsetFileForDisplay.txt');
include('sqliteDB.php');
include('generateBackupSetFiles.php');

# check if not last day for schedule job then exit
if(isset($argv[1]) && "lastDay" === $argv[1] && !checkLastDayOfMonth()){
	traceLog("Cannot start Schedule job, Reason : last day of the month.");
	exit;
}

# Subroutine that processes SIGINT and SIGTERM signal received by the script during backup 
if( extension_loaded('pcntl') && function_exists('pcntl_signal') ){
	declare(ticks = 1);
	pcntl_signal(SIGTERM, 'process_term');// Termination ('kill' was called)
	pcntl_signal(SIGINT, 'process_term'); // Interrupted (Ctrl-C is pressed)
}

checkIfSetEmpty(BACKUPSETFORDISPLAY);

# create pid.txt if required.
createPid($taskType);
$operationName = BACKUP;
# pre-cleanup for all intermediate files and folders.
removeBackupTempFiles();

# To get the Operation type as mirror/relative
getBackupOperationType();

# Define output user log file name and create LOG and ERROR directories
createLogFiles($outputFileName);

# check if binary exist and have execute permission
checkBinaryExists();

# To load the partial and full Exclude array from Exclude file
loadExcludeArray();

writeLogHeader();
if(!preg_match("/$varManualTask/", $jobDir)){
	waitforArchiveJobtoFinish($ArchiveJobPath);	
	waitforDeleteJobtoFinish($DeleteJobPath);	
}

if(!($ERRORFAILEDFILE = fopen($errorFile, "w"))){
	$errStr = "Could not open file $errorFile to write, Reason:$!";
	traceLog($errStr);
	exit_cleanup();
	exit;
}

startBackup();
if($errStr != ""){
	traceLog($errStr);
	writeBackupErrorMsgToStatusFile($errStr);
}
exit_cleanup();


#******************************************************************************************************************************************
# Subroutine Name         : startBackup.
# Objective               : Start backup operation by reading the content of info_file. The info_file is getting created and written by 
#							generateBackupsetFiles() subroutine. Check the retry condition and perform retry to backup, if needed. 
# Added By                : Pooja Havaldar
#******************************************************************************************************************************************
function startBackup()
{
	GLOBAL $FD_WRITE, $BACK_FILE, $FD_READ, $errStr, $pid_generate, $exitStatus, $failedFilesCount;
	GLOBAL $retrycount, $failedfiles_index, $statusFileHash, $jobrunningDir; 
	GLOBAL $bacSync, $totalFiles, $webDir, $APPDIR, $logOutputFilePath;

	# To udpate backupset files to sync with the root(/) directory.
	autoUpdateBackupset();

	#opening info file for generateBackupsetFiles function to write backup set information and for main process to read that information
	if(!($FD_WRITE = fopen(INFOFILE,"w"))){
		$errStr = "Could not open file ".INFOFILE." to write, Reason:".customErrorMsg();
		return;
	}
	
	if( extension_loaded('pcntl') && function_exists('pcntl_fork') ){
		$pid_generate = pcntl_fork();
		if($pid_generate == -1){
			$errStr = "Unable to start generateBackupsetFiles operation. Reason:".customErrorMsg();
			return;
		}

		# this child process will create all backup set files with 1000-1000 files set
		if($pid_generate == 0){
			generateBackupsetFiles();
			exit;
		}
	} 
	else {
		$forkProcessScriptPath = $webDir.PATHSEPARATOR.APPDIR.PATHSEPARATOR.SCRIPTS.PATHSEPARATOR.FORKPROCESSFILE; 
		$cmd = "$phpPath $forkProcessScriptPath 'backup' '$logOutputFilePath'";
		$pid_generate = exec("$cmd > /dev/null 2>".EXEC_ERRORFILE." & echo $!");			
	}
	
	fclose($FD_WRITE);
	fclose($BACK_FILE);
	do
	{	
		if(!($FD_READ = fopen(INFOFILE,"r"))){
			$errStr = "Could not open file ".INFOFILE." to read, Reason:".customErrorMsg();
			return;
		}
		
		#this loop will read info file and for every record it will execute doBackupOperation for backing up the data
		while (1){
			if(! file_exists(PIDFILE))
				break;
			
			$line = fgets($FD_READ);
			$line = trim($line);
			if(empty($line)){
				sleep(1);
				fseek($FD_READ, 0, SEEK_CUR);		#to clear eof flag
				continue;
			}
			
			if(preg_match("/^BAC_SYNC/", $line)){
				$lineSplit = preg_split("/ /", $line,2);
				$bacSync = trim($lineSplit[1]);
				continue;
			}
			
			if(preg_match("/^TOTALFILES/", $line)){
				$lineSplit = preg_split("/ /", $line,2);
				$totalFiles = trim($lineSplit[1]);
				continue;
			}
			
			if(preg_match("/^FAILEDCOUNT/", $line)){
				$lineSplit = preg_split("/ /", $line,2);
				$failedFilesCount = trim($lineSplit[1]);
				break;
			}
			
			if(!doBackupOperation($line)){
				$exitStatus = 1;
				break;
			}
		}
		
		fclose($FD_READ);
		
		# empty statusFileHash array to get latest content.
		$statusFileHash = array();
		waitForProcess($pid_generate,NOSCANERRORFILE);	
				
		if($totalFiles == 0){
			$filecontent = file(INFOFILE);

			for($i = count($filecontent) - 1; $i >= 0; $i--){
				if(preg_match("/^FAILEDCOUNT/", $filecontent[$i])){
					$lineSplit = preg_split("/ /", $filecontent[$i],2);
					$failedFilesCount = trim($lineSplit[1]);
				}
				elseif(preg_match("/^TOTALFILES/", $filecontent[$i])){	
					$lineSplit = preg_split("/ /", $filecontent[$i],2);
					$totalFiles = trim($lineSplit[1]);
				}
				elseif(preg_match("/^BAC_SYNC/", $filecontent[$i])){
					$lineSplit = preg_split("/ /", $filecontent[$i],2);
					$bacSync = trim($lineSplit[1]);
					break;
				}
			}
		
			#loop and traverse from last
			if($totalFiles == 0)
				traceLog("Unable to get total files count");
		}
		
		$retry = 0;
		if(($exitStatus == 0) and ($retrycount < MAX_RETRY) and file_exists(RETRY_INFOFILE) and file_exists(PIDFILE) and customFileSize(RETRY_INFOFILE) > 0){
			$failedfiles_index = 0;
			if(!updateStatusFile($failedfiles_index))
				break;
			
			if(!rename(RETRY_INFOFILE, INFOFILE)){
				traceLog("Unable to rename ".RETRY_INFOFILE." to ".INFOFILE.", Reason: ".customErrorMsg());
				break;
			}
	
			if(!($FD_WRITE = fopen(INFOFILE,"a"))){
				$errStr = "Could not open file ".INFOFILE." to write, Reason:".customErrorMsg();
				break;
			}
			
			fwrite($FD_WRITE, "BAC_SYNC $bacSync\n");
			fwrite($FD_WRITE, "TOTALFILES $totalFiles\n");
			fwrite($FD_WRITE, "FAILEDCOUNT $failedFilesCount");
			fclose($FD_WRITE);
		
			# checking for last retry to instruct child process not to create any more retry info file
			if($retrycount == MAX_RETRY - 1){
				$failedfiles_index = -1;
				if(!updateStatusFile($failedfiles_index))
					break;
			}
			
			shell_exec("rm -rf ".ERROR_DIR);
			$errorFilePath = $jobrunningDir."/".ERROR_DIR;

			if(!is_dir($errorFilePath)){
				if(! mkdir($errorFilePath)){
					traceLog("Unable to create directory $errorFilePath, Reason: ".customErrorMsg());
					break;
				}
				@chmod($errorFilePath, PERMISSIONVAL);
				
				$errorFilePathInter = ERROR_DIR."/".TEMP_FILES;
				if(! mkdir($errorFilePathInter)){
					traceLog("Unable to create directory $errorFilePathInter, Reason:".customErrorMsg());
					exit;
				}
				chmod($errorFilePathInter,PERMISSIONVAL);
			}
			updateRetryCount();
			$retry = 1;
		}
		
	}while($retry == 1);	
}


#****************************************************************************************************
# Subroutine Name         : doBackupOperation.
# Objective               : This subroutine performs the actual task of backing up files. It creates 
#							a child process which executes the backup command. It also creates a process
#							which continuously monitors the temporary output file. At the end of backup, 
#							it inspects the temporary error file if present. It then deletes the temporary 
#							output file, temporary error file and the temporary directory created by 
#							idevsutil binary.             
# Modified By             : Deepak Chaurasia, Pooja Havaldar
#*****************************************************************************************************/
function doBackupOperation($parameters)
{ 
	GLOBAL $isLocalBackup;
	GLOBAL $idevsutilBinaryPath;
	GLOBAL $webDir;
	GLOBAL $logOutputFilePath;
	GLOBAL $backupLocation;
	GLOBAL $operationStartTime;
	GLOBAL $backupUtfFile;
	GLOBAL $errStr;
	GLOBAL $machine;
	GLOBAL $phpPath;

	$parameter_list = array();
	$parameter_list = preg_split("/ /", $parameters,3);
	$opType = BACKUP;
	
	if($isLocalBackup == 1){
		$opType = LOCALBACKUP;	
	} 
	
	$backupUtfFile = getOperationFile($opType,$parameter_list[0] ,$parameter_list[1] ,$parameter_list[2]);			
	# EVS command to execute for backup
	$idevsutilCommandLine = $idevsutilBinaryPath.WHITESPACE.IDEVSUTIL_ARG.ASSIGNMENTOP."\"$backupUtfFile\"";

	if(file_exists(PIDFILE)){
		$pid = exec("$idevsutilCommandLine > /dev/null & echo $!");
		// backup child process for output file parsing 
		$backupLocationSource = base64_encode($backupLocation);
		$fileChildProcessPath = $webDir.PATHSEPARATOR.APPDIR.PATHSEPARATOR.SCRIPTS.PATHSEPARATOR.BACKUP_CHILDPROCESSFILE; 
		$cmd = "$phpPath $fileChildProcessPath \"$logOutputFilePath\" \"$parameter_list[0]\" \"$parameter_list[1]\" \"$parameter_list[2]\" '$backupLocationSource' \"$operationStartTime\"";
		$pid_OutputProcess = exec("$cmd > /dev/null 2>".EXEC_ERRORFILE." & echo $!");
	}
	else
		return 0;

	waitForEVS($pid, SCANERRORFILE);

	# update server address if cmd failed due to wrong evs server address
	checkErrorAndUpdateServerAddr();
	
	if(!($OFH = fopen(OUTPUTFILE,"a"))){
			$errStr = LINEFEED."Failed to open output file. Reason : ".customErrorMsg().LINEFEED;
		traceLog($errStr);
		return 0;
	}
	else{
		fwrite($OFH, "CHILD_PROCESS_COMPLETED".LINEFEED);;
		fclose($OFH);
	}

	waitForEVS($pid_OutputProcess, NOSCANERRORFILE);
	return BACKUP_SUCCESS;
}

#**********************************************************************************************************
# Subroutine Name         : waitForEVS
# Objective               : This function will continuously check for any error in error.txt and exit from childprocess.		
# Added By                : Pooja Havaldar
#**********************************************************************************************************/
function waitForEVS($processPid, $value)
{
	$prevErrorSize = "";
	while(1){
		if(! file_exists("/proc/$processPid")) 
			return;
		
		#Child is still running.
		if($value == SCANERRORFILE){
			if(checkErrorFile($prevErrorSize))
				break;
		}
		sleep(3);
	}
	return;
}

#**********************************************************************************************************
# Subroutine Name         : checkErrorFile
# Objective               : This function will compare error.txt content with error msgs and terminate the job if required.		
# Added By                : Pooja Havaldar
#**********************************************************************************************************/
function checkErrorFile(&$prevErrorSize)
{
	GLOBAL $ErrorArgumentsExit, $fileJobTerminationPath, $phpPath, $errStr;
	
	#checking if error.txt has size
	if(file_exists(ERRORFILE) and customFileSize(ERRORFILE) == 0){
		return 0;
	}
	if(!file_exists(ERRORFILE)){
		return 0;
	}
	
	$currentErrorSize = 0;
	$currentErrorSize = customFileSize(ERRORFILE);	
	if($currentErrorSize == $prevErrorSize){
		return 0;
	}
	
	$linesBackupErrorFile = "";
	$linesBackupErrorFile = file_get_contents(ERRORFILE);
	
	$errorline = "idevs error";
	
	for($j=0; $j <= count($ErrorArgumentsExit)-1; $j++){
			if(preg_match("/$ErrorArgumentsExit[$j]/", $linesBackupErrorFile)){
				checkDedupDeviceError($linesBackupErrorFile);
				$errStr = "Operation could not be completed. Reason : $ErrorArgumentsExit[$j] \n";
				traceLog($errStr);
				$cmd = "$phpPath $fileJobTerminationPath";
				`$cmd`;
				@unlink(ERRORFILE);
				return 1;
			}
		}
	
	$prevErrorSize = $currentErrorSize;
	return 0;
}

#****************************************************************************************************
# Subroutine Name         : waitforArchiveJobtoFinish.
# Objective               : This subroutine will wait till Archive Job finishes
# Modified By             : Pooja Havaldar
#*****************************************************************************************************/
function waitforArchiveJobtoFinish($ArchiveJobPath)
{
	GLOBAL $logOutputFilePath;
	
	$arch_flag = 0;
	while(checkIfJobRunning($ArchiveJobPath)){
		if(!file_exists(PIDFILE)){
			$errMsg = PIDFILE." does not exist for backup.";
			traceLog($errMsg);
			exit_cleanup();
			exit;
		}
		
		if($arch_flag == 0){
			if(!($logOutputHandle = fopen($logOutputFilePath,"a"))){
				traceLog("Could not open file $logOutputFilePath, Reason:".customErrorMsg().LINEFEED);
				sleep(5);
				continue;
			}
			fwrite($logOutputHandle, "Your scheduled job has been delayed, as Archive Cleanup is in progress.".LINEFEED);	
			fclose($logOutputHandle);
			traceLog("Your scheduled job has been delayed, as Archive Cleanup is in progress.");
			$arch_flag = 1;
		}
		sleep(5);
	}	
}

#****************************************************************************************************
# Subroutine Name         : waitforDeleteJobtoFinish.
# Objective               : This subroutine will wait till Delete Job finishes
# Modified By             : Vijay Vinoth
#*****************************************************************************************************/
function waitforDeleteJobtoFinish($DeleteJobPath)
{
	GLOBAL $logOutputFilePath;
	$flag = 0;
	while(checkIfDeleteJobRunning($DeleteJobPath)){
		if(!file_exists(PIDFILE)){
			$errMsg = PIDFILE." is not there in job path.";
			traceLog($errMsg);
			exit_cleanup();
			exit;
		}
		
		if($flag == 0){
			if(!($logOutputHandle = fopen($logOutputFilePath,"a"))){
				traceLog("Could not open file $logOutputFilePath, Reason:".customErrorMsg().LINEFEED);
				sleep(5);
				continue;
			}
			fwrite($logOutputHandle, "Your scheduled job has been delayed, as Delete is in progress.");
			fclose($logOutputHandle);
			traceLog("Your scheduled job has been delayed, as Delete is in progress.", __FILE__, __LINE__);
			$flag = 1;
		}
		sleep(5);
	}	
}

#****************************************************************************************************
# Subroutine Name         : loadExcludeArray
# Objective               : To load the partial and full exclude array from Exclude file
# Added By                : Pooja Havaldar
#*****************************************************************************************************/
function loadExcludeArray()
{
	GLOBAL $excludeFullPath;
	GLOBAL $excludeArray;
	GLOBAL $parStr;
	GLOBAL $backupExcludeHash;
	GLOBAL $excludeParArray;
	GLOBAL $quotemeta;
	GLOBAL $logOutputFilePath;
	GLOBAL $excludePartialPath;
	GLOBAL $hidden;
	
	$errStr = "";
	$excludeArrayInitial = array();
	$excludeParArray = array();
	
	# read full path exclude file and prepare a hash for it
	if(file_exists($excludeFullPath) and customFileSize($excludeFullPath)>0 and file_exists(PIDFILE)){
		$excludeArrayInitial = file($excludeFullPath);
		
		for($i=0; $i<count($excludeArrayInitial); $i+=2){
			$excludeArrayInitial[$i] = trim($excludeArrayInitial[$i]);
			if($excludeArrayInitial[$i] == "")
				continue;
			
			$backupExcludeHash[$excludeArrayInitial[$i]] = 1;
			$excludeArrayInitial[$i].="/";
			array_push($excludeArray, $excludeArrayInitial[$i]);
		}		
	}
	
	#read partial path exclude file and prepare a partial match pattern 
	if(file_exists($excludePartialPath) and customFileSize($excludePartialPath) and file_exists(PIDFILE)){
		$excludeParArray = file($excludePartialPath);			
		if(trim($excludeParArray[0]) == '\/.')
			$hidden = "Disabled";

		for($i = 0; $i < count($excludeParArray); $i++)
			$quotemeta[$i] = phpQuoteMeta(trim($excludeParArray[$i]));
		
		$parStr = join("\n", $quotemeta);  
		$parStr = trim($parStr);
		$parStr = str_replace("\n", "|", $parStr);
	}
}

#****************************************************************************************************
# Subroutine Name         : process_term.
# Objective               : The signal handler invoked when SIGINT or SIGTERM signal is received by the script                 
# Added By				  : Arnab Gupta
# Modified by 			  : Pooja Havaldar
#*****************************************************************************************************/
function process_term($signal)
{
	GLOBAL $statusFileHash, $processName, $pid_generate, $pid_OutputProcess, $errStr, $dbh, $FD_WRITE, $bacSync, $totalFiles, $failedFilesCount, $ERRORFAILEDFILE, $BACK_FILE;

	if($processName == "main"){
		waitForProcess($pid_generate,NOSCANERRORFILE);
		waitForProcess($pid_OutputProcess,NOSCANERRORFILE);
		
		$filecontent = file(INFOFILE);
		for($i = count($filecontent)-1; $i >= 0; $i--){
			if(preg_match("/^FAILEDCOUNT/",$filecontent[$i])){
				$lineSplit = preg_split('/ /', $filecontent[$i],2);
				$failedFilesCount = trim($lineSplit[1]);
			}
			if(preg_match("/^TOTALFILES/",$filecontent[$i])){
				$split_totalFiles = preg_split('/ /',$filecontent[$i],2);
				$totalFiles = trim($split_totalFiles[1]);
			}
			if(preg_match("/^BAC_SYNC/",$filecontent[$i])){
				$lineSplit = preg_split('/ /',$filecontent[$i],2);
				$bacSync = trim($lineSplit[1]);
				break;
			}
		}

		$errStr = "Operation could not be completed. Reason : Process has been killed";
		
		$statusFileHash = array();
		exit_cleanup();
		exit;
	}
	else{
		fwrite($FD_WRITE, "BAC_SYNC $bacSync\nTOTALFILES $totalFiles\nFAILEDCOUNT $failedFilesCount");
		fclose($FD_WRITE);
		fclose($ERRORFAILEDFILE);
		fclose($BACK_FILE);

		if("" != $errStr){
			traceLog($errStr);
			writeBackupErrorMsgToStatusFile($errStr);
		}
		
		# newly added if dbh is defined then only close.
		if($dbh){
			unset($dbh);
		}
		exit;
	}
}
?>