<?php
GLOBAL $webDir,	$APPDIR,	$currentFilePath;
include('sqliteDB.php');
include('generateRestoresetFiles.php');

if( extension_loaded('pcntl') && function_exists('pcntl_signal') ){
	// Subroutine that processes SIGINT and SIGTERM signal received by the script during restore 
	declare(ticks = 1);
	pcntl_signal(SIGTERM, 'process_term');// Termination ('kill' was called)
	pcntl_signal(SIGINT, 'process_term'); // Interrupted (Ctrl-C is pressed)
}

checkIfSetEmpty($restoreDisplayPath);

#create pid.txt if required.
createPid($taskType);
$operationName = RESTORE;

# pre cleanup intermediate file and folder.
removeRestoreTempFiles();
checkBinaryExists();
createLogFiles($outputFileName);
writeLogHeader();
startRestore();

if($errStr != "")
	writeErrorMsgToStatusFile($errStr);

exit_cleanup();


#******************************************************************************************************************
# Subroutine Name         : writeLogHeader.
# Objective               : This subroutine construct log header message and return constructed message.
# Added By                : Vijay Vinoth
#******************************************************************************************************************/
function writeLogHeader()
{
	GLOBAL $headerMsg,	$logOutputFilePath;
	GLOBAL $username,		$hostName;
	$errStr = '';
	# require to open log file to show job in progress as well as to log exclude details
	$logOutputHandle = fopen($logOutputFilePath, 'a+');
	
	if($logOutputHandle)
	{
		# As log file is opened so create logpid file as well 
		$currPath = trim(getcwd());
		if(file_put_contents(LOGPIDFILE,"$currPath/$logOutputFilePath"))
			chmod(LOGPIDFILE,PERMISSIONVAL);
		else {
			$errStr = "Unable to open file ".LOGPIDFILE.", Reason:".customErrorMsg();
			fwrite($logOutputHandle,$errStr.PHP_EOL);
			fclose($logOutputHandle);
			traceLog($errStr);
			exit_cleanup();
			exit;
		}
		chmod($logOutputFilePath,PERMISSIONVAL);
	}
	else {
		$errStr = "Failed to create $logOutputFilePath, Reason:".customErrorMsg();
		traceLog($errStr);
		exit_cleanup();
		exit;
	}
		
	//$headerMsg = "Restore Start Time: ".WHITESPACE.date("D M d H:i:s Y").LINEFEED;
	$headerMsg = "Restore Start Time: ".WHITESPACE.trim(shell_exec('date +"%a %b %d %T %Y"')).LINEFEED;
	$logheader = $headerMsg.LINEFEED."User name: ".$username.LINEFEED;
	
	$headerMsg .= getHeaderMsg();
	$headerMsg .= "Restore Location: $hostName".LINEFEED;	
	
	$logheader .= getHeaderMsg();
	$logheader .= "Restore Location: $hostName".LINEFEED.LINEFEED;	

	# write log header.
	fwrite($logOutputHandle,$logheader.PHP_EOL);
	if($logOutputHandle)
		fclose($logOutputHandle);
}

#****************************************************************************************************
# Subroutine Name         : startRestore.
# Objective               : Start restore operation by reading the content of info_file.
#							The info_file is getting created and written by generateRestoresetFiles() 
#							subroutine. Check the retry condition and perform retry to restore,
#							if needed. 
# Added By                : Avinash Kumar.
#*****************************************************************************************************/
function startRestore($startRestoreInitiateVar = 0)
{
	GLOBAL $retrycount;
	GLOBAL $totalFiles, $startRestore;
	GLOBAL $current_source, $jobrunningDir,	$statusFileHash;
	GLOBAL $phpPath, $webDir, $APPDIR, $logOutputFilePath;
	
	# creating temp folder for storing intermediate restore set files and info file
	if(!is_dir(TEMP_FILES)){
		if(!mkdir(TEMP_FILES)){
			$errStr = "Unable to create directory ".TEMP_FILES.", Reason:".customErrorMsg();
			exitError($errStr);
			exit;
		}
		@chmod(TEMP_FILES,PERMISSIONVAL);
	}
	
	# opening the info file for transferring the backup set information from child process to main process
	if($startRestoreInitiateVar == 0){
		$ourFileHandle = fopen(INFOFILE, 'w+');
		if(!$ourFileHandle){
			$errStr = "Could not open file ".INFOFILE." to write, Reason:".customErrorMsg();
			traceLog($errStr);
			return;
		}
		@chmod(INFOFILE,PERMISSIONVAL);
		fclose($ourFileHandle);
		if( extension_loaded('pcntl') && function_exists('pcntl_fork') ){			
			$pidGen = pcntl_fork(); 
			if($pidGen == -1)
			{
				$errStr = "Unable to start generateRestoresetFiles operation";
				traceLog($errStr);
				return;
			}
			if($pidGen == 0)
			{
				generateRestoresetFiles();
				exit;
			}
		} else {
			$forkProcessPath = $webDir.PATHSEPARATOR.APPDIR.PATHSEPARATOR.SCRIPTS.PATHSEPARATOR.FORKPROCESSFILE; 
			$cmd = "$phpPath $forkProcessPath restore $logOutputFilePath";
			$pidGen = exec("$cmd > /dev/null 2>".EXEC_ERRORFILE." & echo $!");		
		}
	}
	
	while (1)
	{
		clearstatcache();
		if(!file_exists($processIdFile))
			break;

		if(!file_exists(INFOFILE) || filesize(INFOFILE)<=0){
			sleep(1);					
			continue;
		}	
		break;
	}
	
	$fdRead = fopen(INFOFILE,"r+");
	if(!$fdRead){
		$errStr = "Could not open file ".INFOFILE." to read, Reason:".customErrorMsg();
		traceLog($errStr);
		return;
	}
	
	while (1)
	{
		if(!file_exists(PIDFILE))
			break;
		
		$line = fgets($fdRead);
		if($line == ""){
			sleep(1);
			fseek($fdRead, 0, 1);		#to clear eof flag
			continue;
		}
		
		if(preg_match('/^TOTALFILES/', $line) === 1)
		{
			$lineSplit 	= explode(' ', $line,2);
			$totalFiles = trim($lineSplit[1]);
			break;
		}
		else 
		{
			$line 				= trim($line);
			if(!doRestoreOperation($line)){
				$exitStatus = 1;
				break;
			}
		}
	}
		
	$line = fgets($fdRead);
	if(preg_match('/^FAILEDCOUNT/', $line) === 1){
		$lineSplit 			= explode(' ', $line,2);
		$failedFilesCount 	= trim($lineSplit[1]);
	}
	fclose($fdRead);
	
	# empty statusFileHash array to get latest content.
	$statusFileHash = array();
	$Synccount 		= getParameterValueFromStatusFile("SYNC_COUNT_FILES_INDEX");
	waitForProcess($pidGen,NOSCANERRORFILE);
			
	if($totalFiles == 0){	
		$fdRead = fopen(INFOFILE,"r");
		if(!$fdRead){
			$errStr = "Could not open file ".INFOFILE." to read, Reason:".customErrorMsg();
			traceLog($errStr);
			return;
		}
		
		fclose($fdRead);
		$file_content = file(INFOFILE);
			
		foreach($file_content as $file_content_data)
		{
			if(preg_match('/^TOTALFILES/', $file_content_data) === 1){
				$split_totalFiles 	= explode(' ',$file_content_data,2);
				$totalFiles 		= trim($split_totalFiles[1]);
			}
			
			if(preg_match('/^FAILEDCOUNT/', $file_content_data) === 1){
				$lineSplit 			= explode(' ', $file_content_data,2);
				$failedFilesCount 	= trim($lineSplit[1]);
			}
		}
		
		if($totalFiles == 0)
			traceLog("Unable to find total file count from info file.");
	} 
		
	if(file_exists(RETRY_INFOFILE) && @customFileSize(RETRY_INFOFILE) > 0 && $retrycount < MAX_RETRY && file_exists(PIDFILE) && $exitStatus == 0)
	{
		$failedfiles_index 		= 0;
		if(!updateStatusFile($failedfiles_index)){
			traceLog("update Status file function failed.");
			return;
		}

		rename(RETRY_INFOFILE, INFOFILE);
		if (file_exists(INFOFILE))
		{
			$fd_write			= fopen(INFOFILE, 'a');
			if(!$fd_write){
				$errStr = "Could not open file ".INFOFILE." to read in retry case, Reason:".customErrorMsg();
				traceLog($errStr);
				return;
			}
			fwrite($fd_write, "TOTALFILES $totalFiles\n FAILEDCOUNT $failedFilesCount");
			fclose($fd_write);
			@chmod(INFOFILE,PERMISSIONVAL);
		}
			
		if($retrycount == MAX_RETRY - 1){
			$failedfiles_index 	= -1;
			if(!updateStatusFile($failedfiles_index)){
				traceLog("update Status file function failed.");
				return;
			}
		}
		shell_exec("rm -rf ". ERROR_DIR);				
		$errorFilePath = $jobrunningDir.PATHSEPARATOR.ERROR_DIR;

		if(!is_dir($errorFilePath)){
			if(! mkdir($errorFilePath,PERMISSIONVAL)){
				traceLog("Unable to create directory $errorFilePath, Reason: ".customErrorMsg());
				return;
			}
			
			$errorFilePathInter = ERROR_DIR.PATHSEPARATOR.TEMP_FILES;
			if(! mkdir($errorFilePathInter,PERMISSIONVAL)){
				traceLog("Unable to create directory $errorFilePathInter, ".customErrorMsg());
				exit;
			}
		}
			
		updateRetryCount();
		startRestore(1);
	}
}

#****************************************************************************************************
# Subroutine Name         : doRestoreOperation.
# Objective               : This subroutine performs the actual task of backing up files. It creates 
#							a child process which executes the restore command. It also creates a process
#							which continuously monitors the temporary output file. At the end of restore, 
#							it inspects the temporary error file if present. It then deletes the temporary 
#							output file, temporary error file and the temporary directory created by 
#							idevsutil binary.             
# Modified By             : Vijay Vinoth
#*****************************************************************************************************/
function doRestoreOperation($line)
{ 
	GLOBAL $isLocalRestore,	$idevsutilBinaryPath;
	GLOBAL $webDir,				$phpPath;
	GLOBAL $logOutputFilePath,	$filesonly,			$operationStartTimeRestore;
	GLOBAL $errStr,				$restoreUtfFile,			$retrycount;
	
	$opType 		= RESTORE;
	$parameters 	= $line;
	$parameter_list = explode(' ', $parameters, SPLIT_LIMIT_INFO_LINE);
	
	if($isLocalRestore == 1)
		$opType = LOCALRESTORE;	
	
	$source 				= isset($parameter_list[2])?$parameter_list[2]:'/';
	$restoreUtfFile 		= getOperationFile($opType,$parameter_list[0] ,$parameter_list[1] ,$source);
	# EVS command to execute for restore
	$idevsutilCommandLine 	= $idevsutilBinaryPath.WHITESPACE.IDEVSUTIL_ARG.ASSIGNMENTOP."\"$restoreUtfFile\"";
		
	if(file_exists(PIDFILE))
	{
		$pid = exec("$idevsutilCommandLine > /dev/null 2>".EXEC_ERRORFILE." & echo $!");	
		
	}
	if(file_exists(PIDFILE))
	{
		# restore child process for output file parsing 
		$fileChildProcessPath 	= $webDir.PATHSEPARATOR.APPDIR.PATHSEPARATOR.SCRIPTS.PATHSEPARATOR.RESTORE_CHILDPROCESSFILE; 
		
		# fix for source which contains special characters like "", '' etc.
		if($retrycount == 0)
			$source 			= phpQuoteMeta($source);
		$childCmd 				= "$phpPath $fileChildProcessPath \"$logOutputFilePath\" \"$parameter_list[0]\" \"$parameter_list[1]\" \"$source\" \"$filesonly\" \"$operationStartTimeRestore\"  > /dev/null 2>".EXEC_ERRORFILE." & echo $!";
		$pid_OutputProcess 		= exec($childCmd);		
	}

	waitForEVS($pid,SCANERRORFILE);

	# update server address if cmd failed due to wrong evs server address
	checkErrorAndUpdateServerAddr();
	$outputFile	= fopen(OUTPUTFILE, 'a+');
	if($outputFile)
	{
		fwrite($outputFile,"CHILD_PROCESS_COMPLETED".PHP_EOL);
		fclose($outputFile);
	}
	else 
	{
		traceLog("Unable to open file ".OUTPUTFILE.", Reason: ".customErrorMsg());
		return 0;
	}
	
	waitForEVS($pid_OutputProcess, NOSCANERRORFILE);	
	return RESTORE_SUCCESS;
}

#****************************************************************************************************
# Subroutine Name         : waitForEVS.
# Objective               : This function will continuously check for any error in error.txt and exit from childprocess.
# Added By                : Vijay Vinoth.
#*****************************************************************************************************/
function waitForEVS($processPid, $value)
{
	$prevErrorSize = "";
	while(1)
	{
		if(!file_exists("/proc/$processPid")) 
			break;
		
		#Child is still running.
		if($value == SCANERRORFILE){
			if(checkErrorFile($prevErrorSize))
				break;
		}
		sleep(3);
	}
	return;
}

#****************************************************************************************************
# Subroutine Name         : checkErrorFile.
# Objective               : This function will compare error.txt content with error msgs and terminate the job if required.
# Added By                : Vijay Vinoth.
#*****************************************************************************************************/
function checkErrorFile()
{
	GLOBAL $ErrorArgumentsExit,	$phpPath,	$fileJobTerminationPath;
	GLOBAL $dedupDetailFile, $dedupRestoreDeviceID;
	
	#checking if error.txt has size
	if(!customFileSize(ERRORFILE))
	{
		return 0;
	}
	$errorFileContents 					= file(ERRORFILE);
	if(!$errorFileContents)
	{
		traceLog("Unable to open file ".ERRORFILE.", Reason:".customErrorMsg());
		return 0;
	}
	
	$linesRestoreErrorFile 		= $errorFileContents;
	$errorline 					= "idevs error";
	$linesRestoreErrorFileCount = count($linesRestoreErrorFile);
	
	for($i = 0; $i<= $linesRestoreErrorFileCount; $i++)
	{
		if($linesRestoreErrorFile[$i] == "" or strpos( $linesRestoreErrorFile[$i], $errorline ) !== false)
		{
			continue;
		}
		$ErrorArgumentsExitCount = count($ErrorArgumentsExit);
		
		for($j=0; $j<=$ErrorArgumentsExitCount; $j++)
		{
			if(strpos( $linesRestoreErrorFile[$i], $ErrorArgumentsExit[$j] ) !== false)
			{
				checkDedupDeviceError($linesRestoreErrorFile[$i]);
				$errStr = "Operation could not be completed. Reason : $ErrorArgumentsExit[$j] \ntotalFiles : $totalFiles \n";
				exec("$phpPath $fileJobTerminationPath");
				unlink(ERRORFILE);
				return 1;
			}
		}	
	}
	return 0;
}

#****************************************************************************************************
# Subroutine Name         : appendEnumerationError.
# Objective               : Enumeration of a folder from restore set file gets fail then write 
#							proper error message to log file.
# Added By                : Vijay Vinoth.
#*****************************************************************************************************/
function appendEnumerationError($fileName)
{
	GLOBAL $jobrunningDir,		$outputErrorFile,	$errorFile;
	GLOBAL $failedFilesCount,	$totalFiles;
	
	$errorFile = $jobrunningDir.PATHSEPARATOR.ERROR_DIR.PATHSEPARATOR.TEMP_FILES.PATHSEPARATOR.FAILEDFILES;	
	
	if($outputErrorFile)
		//fwrite($outputErrorFile,"[".date("D M d H:i:s Y")."] [FAILED] [$fileName] Reason : No such file or directory.".LINEFEED);
		fwrite($outputErrorFile,"[".trim(shell_exec('date +"%a %b %d %T %Y"'))."] [FAILED] [$fileName] Reason : No such file or directory.".LINEFEED);
	
	$failedFilesCount++;
	$totalFiles++;
}

#****************************************************************************************************
# Subroutine Name         : exit_cleanup.
# Objective               : This function will execute the major functions required at the time of exit 
# Added By                : Deepak Chaurasia
# Modified By			  : Avinash Kumar
#*****************************************************************************************************/
function exit_cleanup()
{	
	GLOBAL $jobrunningDir,	$restoreUtfFile,	$errStr;
	GLOBAL $successFiles, 	$syncedFiles, 		$failedFiles,	$cancelFlag;
	GLOBAL $errStr,			$cancelFlag;
	
	$successFiles 	= getParameterValueFromStatusFile("RESTORE_COUNT_FILES_INDEX");
	$syncedFiles 	= getParameterValueFromStatusFile("SYNC_COUNT_FILES_INDEX");
	$failedFiles 	= getParameterValueFromStatusFile("FAILED_COUNT_FILES_INDEX");
	$restoreExit 	= getParameterValueFromStatusFile("RESTORE_TERMINATION_INDEX");

	if(file_exists(PIDFILE))
	{
		unlink(PIDFILE);
		if(file_exists(EVS_TEMP))
		{
			exec("rm -rf ".EVS_TEMP);
		}
	}
	else 
	{
		# In childprocess, if we exit due to some exit scenario, then this exit_flag will be true with error msg
		$exit = explode("-",$restoreExit,2);
		
		$cancelFlag = true;
		if(!$exit[0])
		{
			$errStr = readFromUserCancel();			
		}
		else
		{
			if($exit[1] != "")
			{
				$errStr = $exit[1];
				checkDedupDeviceError($errStr);
			}
		}
		traceLog($errStr);
	}
	writeOperationSummary(RESTORE);
	@unlink($jobrunningDir."/error.txt");
	getOperationStatus();
	updateLogFileName();
	sendMail();
	@unlink($restoreUtfFile);
	@unlink(OUTPUTFILE);
	removeRestoreTempFiles(); 
	//@fclose($fhsPid);
	@unlink(STARTPIDFILE);
	@unlink(CANCEL_PATH); 
}

#****************************************************************************************************
# Subroutine Name         : process_term.
# Objective               : The signal handler invoked when SIGINT or SIGTERM signal is received by the script                 
# Added By				  : Arnab Gupta
# Modified by 			  : Vijay Vinoth
#*****************************************************************************************************/
function process_term($signal)
{
	GLOBAL $isErrorFileOpen,	$processName,		$pid_generate,	$pid_OutputProcess;
	GLOBAL $errStr, 			$dbh,			$FD_WRITE;
	GLOBAL $bacSync, 			$statusFileHash,	$totalFiles, 	$failedFilesCount; 
	GLOBAL $outputFile, 		$openRestoreFileOrg,$filehandle,	$outputErrorFile;
	
	if($processName == "main"){
		waitForProcess($pid_generate,NOSCANERRORFILE);
		waitForProcess($pid_OutputProcess,NOSCANERRORFILE);

		$filecontent = file(INFOFILE);
		for($i = count($filecontent)-1; $i >= 0; $i--){
			if(preg_match("/^FAILEDCOUNT/",$filecontent[$i])){
				$lineSplit = preg_split('/ /', $filecontent[$i],2);
				$failedFilesCount = $lineSplit[1];
				$failedFilesCount = trim($failedFilesCount);
			}
			if(preg_match("/^TOTALFILES/",$filecontent[$i])){
				$split_totalFiles = preg_split('/ /',$filecontent[$i],2);
				$totalFiles = $split_totalFiles[1];
				$totalFiles = trim($totalFiles);
			}
		}

		$errStr = "Operation could not be completed. Reason : Process has been killed";
		
		$statusFileHash = array();
		exit_cleanup();
		exit;
	}
	else{
		fwrite($FD_WRITE, "TOTALFILES $totalFiles\n");
		fwrite($FD_WRITE, "FAILEDCOUNT $failedFilesCount");
		fclose($FD_WRITE);
		
		if($isErrorFileOpen){
			@fclose($outputErrorFile);
		}
		
		if($outputFile){
			@fclose($outputFile);
		}
		
		if($openRestoreFileOrg){
			@fclose($openRestoreFileOrg);
		}
		
		if($filehandle){
			@fclose($filehandle);
		}
		# close DB connection for offline restore
		if($dbh_LB){
			unset($dbh_LB);
		}
		
		if($errStr != ""){
			# write $errStr to status file.
			writeErrorMsgToStatusFile($errStr);
		}
	}
}
?>