<?php
define('APPDIR', 'IDrive');
define('SERVICE', 'idriveIt');
define('SYNOLOGY','synology');
define('QNAP','qnap');
define('NETGEAR','netgear');
define('TWOTIER','twotier');
define('ASUSTOR','Asustor');

if(!isset($argv[1]) || !trim($argv[1])){
	exit('Enter machine name');
}

$machine_type = $argv[1];
$webDirPath = "";

if($machine_type == SYNOLOGY){
	$webDirPath = "/var/services/web/";
}
else if($machine_type == QNAP){
	$webDirPath = "/share/Web/";
}
else if($machine_type == NETGEAR){
	$webDirPath = "/apps/idrive/web/";
}
else if($machine_type == TWOTIER){
	$webDirPath = "/usr/local/www/";
}
else if($machine_type == ASUSTOR){
	$webDirPath = "/usr/local/www/";
	$binaryPath = $webDirPath.APPDIR;
	doCleanup();
}

$binaryPath = $webDirPath.APPDIR;


if(($machine_type == NETGEAR) || ($machine_type == QNAP)){

	validateIdevsutil();
	if($machine_type == QNAP){
		$suPath = $binaryPath."/.config/su";
		if(!validateSu($suPath)){
			system("sh $binaryPath/.scripts/suBinaryChk.sh $binaryPath/.config $machine_type");
		}
	}
	
}
else if($machine_type == SYNOLOGY){

	$unameOut = `uname -m`;
	$unameOut = trim($unameOut);
	$orgLibDir = $webDirPath.APPDIR."/.config";
	$orgLib  = "$orgLibDir/libpopt.so.0.0.0";
	$libPath = "/lib";	
	if("x86_64" == $unameOut)
		$libPath = "/lib32";
	
	$libPathPath = "$libPath/libpopt.so.0";
	if(!file_exists($libPathPath)){
		getCompatibleLibpopt($unameOut,$orgLibDir);
		
		if(is_link($libPathPath)){
			unlink($libPathPath);
		}
		`ln -s $orgLib $libPathPath`;
	}
	else if(is_link($libPathPath)){
		if(!file_exists(readlink($libPathPath))){
			getCompatibleLibpopt($unameOut,$orgLibDir);		
		}
		if(is_link($libPathPath)){
			unlink($libPathPath);
		}
		`ln -s $orgLib $libPathPath`;
	}

	$dsmVersion6 = isDSM6();
	doCleanup($dsmVersion6);
}

/*********************************************************************************************************
* Subroutine Name       : isDSM6
* Objective             : To find, is this DSM 6
* Added By              : Sabin Cheruvattil | 2016-11-25
**********************************************************************************************************/
function isDSM6()
{
	$dsmVersionArr = explode("\n", `cat /etc.defaults/VERSION`);
	array_walk($dsmVersionArr, 'trim');
	
	if(preg_match("/\"6\"$/", $dsmVersionArr[0])) {
		return TRUE;
	}	
	return FALSE;
}

/*********************************************************************************************************
* Subroutine Name       : doCleanup
* Objective             : Restart php-fpm & httpd servers.
* Added By              : Sabin Cheruvattil | 2016-11-28
**********************************************************************************************************/
function doCleanup($isDSM6=0)
{
	GLOBAL $phpFpmConfPath, $phpNameArray, $machine_type;
	# This is still a guess, we will try to find the actual path to it.
	getPHPConfdPath($isDSM6);

	foreach($phpFpmConfPath as $additionalConfPath){
		$additionalConfPath = trim($additionalConfPath);
		if($additionalConfPath == '') {
			continue;
		}
		if($machine_type == ASUSTOR){
			$orgIni = "/usr/local/AppCentral/".APPDIR."/www/.scripts/".APPDIR.".ini";
		}else{
			$orgIni = "/var/packages/".APPDIR."/target/".APPDIR.".ini";
		}
		$newIni = $additionalConfPath."/".APPDIR.".ini";
		if(is_link($newIni)){
			unlink($newIni);
		}
		`ln -s $orgIni $newIni`;
	}
	
	
	if($machine_type == ASUSTOR){
		`/usr/builtin/sbin/apache2 -k restart`;
	}else if ($isDSM6) {
		foreach($phpNameArray as $phpversion) {
			if ($phpversion == "php") {
				`initctl restart $phpversion-fpm`;
			}
			else {
				`initctl restart pkg-$phpversion-fpm`;
			}
		}
		`/usr/sbin/initctl restart pkg-apache22`;
	}
	else {
		`/sbin/initctl stop httpd-user`;
		`/sbin/initctl start httpd-user`;
	}

	validateIdevsutil();
	exit;
}


/*************************************************************************************************************************
* Subroutine Name       : validateIdevsutil
* Objective             : validate idevs util binary file. If it is invalid, downloads the appropriate one from server
**************************************************************************************************************************/
function validateIdevsutil(){

	GLOBAL $binaryPath, $machine_type;
	#$lines = 0;
	$res = '';
	#Checking the validity of idevsutil binary
	$idevsutil = $binaryPath."/idevsutil_IDrive";
	if(file_exists($idevsutil)){
		#$lines = `$idevsutil -h 2>>/dev/null | wc -l`;
		$res = `$idevsutil --client-version | tr -d '\r\n'`;
	}
	#if($lines < 50){
	if(strpos($res,"Capabilities: 64-bit files, socketpairs")===false){
		system("sh $binaryPath/.scripts/binaryChk.sh $idevsutil $machine_type 2>/dev/null 1>2");
	}
}


/*********************************************************************************************************
* Subroutine Name       : getPHPConfdPath
* Objective             : To get the php conf.d path
* Added By              : Sabin Cheruvattil | 2016-11-25
**********************************************************************************************************/
function getPHPConfdPath($isDSM6=0)
{
	GLOBAL $phpFpmConfPath, $phpNameArray, $machine_type;
	$phpFpmConfPath = array();
	$psPath				= "ps -wwwww";

	if($isDSM6 or $machine_type == ASUSTOR) {
		$psPath 		= "ps -elf";
	}

	$phpFpmPath    		= `$psPath | grep 'php-fpm' | grep 'master process'`;
	preg_match_all("/\/(php.*)\/fpm.conf/", $phpFpmPath, $match);
	
	$phpNameArray = $match[1];
	if(!in_array('php',$phpNameArray)) {
		array_push($phpNameArray,'php');
	}
	
    foreach($phpNameArray as $phpName){	
		$pathToPhpBinary 	= trim(`which $phpName`);
		if($pathToPhpBinary != '') {
			$phpIniInfo = `$pathToPhpBinary --ini`;
			$output  	= explode("\n", $phpIniInfo);
			foreach($output as $value) {
				if(preg_match("/^Scan for additional .ini files in:/", $value)) {
						$iniDirPath = trim(preg_replace("/Scan for additional .ini files in:/", '', $value));
						if(strstr($iniDirPath,'/'))
							array_push($phpFpmConfPath,$iniDirPath);
					break;
				}
			}
		}
	}	
}

/***********************************************************************************************
* Subroutine Name       : getCompatibleLibpopt
* Objective             : get the appropriate libpopt file based on machine arch
***********************************************************************************************/
function getCompatibleLibpopt($arch="",$basePath="")
{
	$libpoptPath = $basePath."/libpopt/libpopt.so.0.0.0";
	$archBaseLibpoptPath = "";
	if(preg_match("/armv/i",$arch)){
		$cmd = `uname -a `;
		if(preg_match("/alpine/i", $cmd)){
			$archBaseLibpoptPath = $libpoptPath."_alpine";	
		}
		else{
			$archBaseLibpoptPath = $libpoptPath."_arm";			
		}
		
	}
	else{
		$archBaseLibpoptPath = $libpoptPath."_X86";
	}

	if(file_exists($archBaseLibpoptPath)){
		$orgLibPath = $basePath."/libpopt.so.0.0.0";
		`mv $archBaseLibpoptPath $orgLibPath`;
		`chmod 00755 $orgLibPath`;
		`chown root:root $orgLibPath`;
	}
}

/********************************************************************************************
* Subroutine Name       : validateSu
* Objective             : validates su binary file 
*********************************************************************************************/
function validateSu($suPath=""){
	$outfile = "/tmp/out.txt";
	$suHelpRes = `$suPath --help  1>$outfile 2>>$outfile`;
	$suLines=`wc -l /tmp/out.txt | awk -F " " '{print $1}'`;
	if(file_exists($outfile)){
		unlink($outfile);
	}
	if($suLines < 5){
		return FALSE;
	}
	return TRUE;
}
